/*************************************************************************
*									 *
*  Pmod_ISNS20_Simple_Example.pde                                        *
*  - Example Sketch for Pmod ISNS20                                      *
*									 *
*  Author(s):	Erik Cegnar / Matt Foreman / Sudharsan Sukumar		 *
*  Copyright (c) 2016, Digilent Inc.  	    				 *
*************************************************************************/
/*

  This project is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This project is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this project; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/
/************************************************************************/
/*  Module Description:							*/
/*									*/
/*  This sketch is an example on on how to read the current values      */
/*  from the Pmod ISNS20.                                               */
/************************************************************************/
/*  Revision History:							*/
/*									*/
/*  3/1/2016 (Erik C / Matt F / Sudharsan Sukumar): Created             */
/************************************************************************/
/*  Board Support: All							*/
/*									*/
/*  Example Board:                                                      */
/*  chipKit Uno32/uC32                CS Pin  = pin 7                   */
/*                                    CLK Pin = pin 13                  */
/*                                    DO Pin  = pin 12                  */
/*			              *NOTE: Use 3.3v source for VCC    */
/*                                                                      */
/************************************************************************/
/*  Parameters:  							*/
/*									*/
/*  FILTER_FLAG:                  1 = Filter on / 0 = No filter         */
/*  NUM_READINGS:                 Number of values to be averaged       */
/*                                in filter.                            */
/*  Jumper Settings:                    				*/
/*									*/
/*  JP4: RG9/DIGITAL - Short middle and right pins              	*/
/*  JP5/JP7: MASTER - Short middle and top pins				*/
/*									*/
/*									*/
/************************************************************************/

/* -------------------------------------------------------------------- */
/*              Include File Definitions                                */
/* -------------------------------------------------------------------- */
#include <DSPI.h>

/* -------------------------------------------------------------------- */
/*              Local Type Definitions                                  */
/* -------------------------------------------------------------------- */

//chip select pin on the chipKIT uC32 (board dependent)
#define CS_PIN 7 

//percent correction for sensitivity error (e.g. 100 = 100%)
//**user can use reference current to calibrate this value precisely
#define SENSITIVITY_CORRECTION 100 

//enable/disable digital filter                                    
#define FILTER_FLAG 1  

//number of samples for filter
//**lower amount of filter readings for a faster reading
#define NUM_READINGS 5000           

/* -------------------------------------------------------------------- */
/*              Forward Declarations                                    */
/* -------------------------------------------------------------------- */
signed int get_mA(void);
float get_A(void);

/* -------------------------------------------------------------------- */
/*              Initialization and Setup Functions		        */
/* -------------------------------------------------------------------- */
DSPI0 spi;

void setup() {
 
  Serial.begin(9600);
   
  spi.begin();
  
  pinMode(CS_PIN, OUTPUT);
}

/* -------------------------------------------------------------------- */
/*              Main Loop                        		        */
/* -------------------------------------------------------------------- */
void loop() {

  signed int ma = 0;
  float a = 0.0;
  
  ma = get_mA();
  Serial.print(ma);               //prints mA value over serial monitor
  Serial.print(" mA   |  ");
  
  a = get_A();
  Serial.print(a);                //prints amp value over serial monitor
  Serial.println(" A");
  
  if(FILTER_FLAG == 0){
    delay(50);
  }
}

/************************************************************************/
/*  signed int get_mA(void)    						*/
/*                                                                      */
/*      Return Value:                                                   */ 
/*              float                                                   */
/*                                                                      */
/*      Description:                                                    */
/*              This function returns the value of current flow in      */
/*              milliamperes (milliamps).                               */
/*                                                                      */
/************************************************************************/
signed int get_mA(void)
{
  int result1;                 //first byte from ADC
  int result2;                 //second byte from ADC
  signed int temp_value;                  
  signed int total = 0;        //running total of filter values
  signed int milli_amps;       //converted value into milli amperes  
  
  if(FILTER_FLAG == 1){
    for(int i; i < NUM_READINGS; i++){
      digitalWrite(CS_PIN, LOW);      //begin SPI transfer
      result1 = spi.transfer(0x00);   //transfer first byte in
      result2 = spi.transfer(0x00);   //transfer second byte in
      digitalWrite(CS_PIN, HIGH);     //end transfer
    
      temp_value = (SENSITIVITY_CORRECTION / 100) * (10000  * (((result1<<8) | result2) - 2048)) / 819;  //appends the two bytes and converts their value into milliamperes
      total = total + temp_value;
    }
    milli_amps = total / NUM_READINGS; 
  }
  
  else{
    digitalWrite(CS_PIN, LOW);      //begin SPI transfer
    result1 = spi.transfer(0x00);  //transfer first byte in
    result2 = spi.transfer(0x00);  //transfer second byte in
    digitalWrite(CS_PIN, HIGH);     //end transfer
    
    milli_amps = (SENSITIVITY_CORRECTION / 100) * (10000  * (((result1<<8) | result2) - 2048)) / 819;  //appends the two bytes and converts their value into milliamperes
  }
  return milli_amps;
}

/************************************************************************/
/*  float get_A(void)    						*/
/*                                                                      */
/*      Return Value:                                                   */ 
/*              float                                                   */
/*                                                                      */
/*      Description:                                                    */
/*              This function returns the value of current flow in      */
/*              amperes (amps).                                         */
/*                                                                      */
/************************************************************************/
float get_A(void)
{
  int result1;           //first byte from ADC
  int result2;           //second byte from ADC
  float temp_value;      
  float total = 0;       //running total of filter values
  float amps;            //converted value into amperes
  
  if(FILTER_FLAG == 1){
    for(int i; i < NUM_READINGS; i++){
      digitalWrite(CS_PIN, LOW);      //begin SPI transfer
      result1 = spi.transfer(0x00);   //transfer first byte in
      result2 = spi.transfer(0x00);   //transfer second byte in
      digitalWrite(CS_PIN, HIGH);     //end transfer
    
      temp_value = (SENSITIVITY_CORRECTION / 100) * (10.0 * (((result1<<8) | result2) - 2048.0) / 819.0);  //appends the two bytes and converts their value into amperes
      total += temp_value;
    }
    amps = total / NUM_READINGS; 
  }
  
  else{
    digitalWrite(CS_PIN, LOW);      //begin SPI transfer
    result1 = spi.transfer(0x00);  //transfer first byte in
    result2 = spi.transfer(0x00);  //transfer second byte in
    digitalWrite(CS_PIN, HIGH);     //end transfer
    
    amps = (SENSITIVITY_CORRECTION / 100) * (10.0 * (((result1<<8) | result2) - 2048.0) / 819.0);  //appends the two bytes and converts their value into amperes
  } 
  return amps;
}
